try:
	from pygext.gl.all import *
	from pygext.lazy import Random as lazyRand
except:
	print "[error] Please install the pygext library"
	print "[pygext-url] http://codereactor.net/projects/pygext/"
	print "[pygext-info] If making source-install, you will also need Polygon lib"
	print "[polygon-url] http://www.dezentral.de/soft/Polygon/"
	
import os, sys, string
from math import pi, sin, cos, atan2, degrees, pow, sqrt, asin
import random

try:
	import pygame
	import pygame.time
	from pygame.constants import *
except:
	print "[error] Please install the pygame library"




# Default config options
screen_w = 1024
screen_h = 768
psyco_on = True
fs = True
sound = True

WinSpawnRate = 240 # Empirical edge of madness
StartSpawnRate = 700
RateTimeDecrease = 1.0 # msec / second
MaxBullets = 7

# Parse command-line options
for opt in sys.argv[1:]:
	if opt == "nofs":
		fs = False
	
	if opt == "noaccel":
		psyco_on = False
	
	if opt == "nosound":
		sound = False
		
	elif opt == "800":
		screen_w, screen_h = 800, 600
		
	elif opt.find("bullets") >= 0:
		 MaxBullets = int( string.split(opt, "=")[1] )
			
	elif opt.find("startrate") >= 0:
		StartSpawnRate = int( string.split(opt, "=")[1] )
	
	elif opt.find("endrate") >= 0:
		WinSpawnRate = int( string.split(opt, "=")[1] )
	
	elif opt.find("deltarate") >= 0:
		RateTimeDecrease = int( string.split(opt, "=")[1] )
	
	
# Usage
print "Darkest Darkness by Mindless Game Studio"
print "----------------------------------------"
print "Options:"
print " nofs    : windowed mode"
print " 800     : 800x600 mode (default: 1024x768)"
print " noaccel : psyco is turned off"
print " nosound : no sound effects"
print ""
print "These options are for advanced occassional gameplay:"
print " startrate=[rate] : average interval between monster spawnings (default 700)"
print " endrate=[rate]   : average interval between monster spawnings to win (default 240)"
print " deltarate=[rate] : rate decrease under one second"
print " bullets=[number] : number of bullets (default 7)"
print ""
print "Enjoy :)"

# To avoid "slow monsters" in higher res. than 800x600
pixelratio = screen_w/800.0

BulletDamage = 1500.0 # Ultra-kill value
ReloadTime = 1000.0 #msec
FlashFadeout = 1000.0 #also see the curve defined in a function called FlashFunc or something..
NormalSpeed = 30.0 # Animation density will be relative to this constant

NumAnims = 4
CorpseDelay = 4.0
AlphaFadeTime = 5.0

DataPath = "data"
MovePostfix = "_move_"
DiePostfix = "_die"
ImgExt = ".png"

BulletImg = "bullet"
BloodImg = "blood"
HpImg = "hp"

monsterStartPos = [(x, -30) for x in range(-30, screen_w + 30, 30)]
monsterStartPos.extend( [(x, screen_h + 30) for x in range(-30, screen_w + 30, 30)] )
monsterStartPos.extend( [(-30, y) for y in range(-30, screen_h + 30, 30)] )
monsterStartPos.extend( [(screen_w+30, y) for y in range(-30, screen_h + 30, 30)] )

############
## utils
############

class Dummy:
	def play(self):
		pass

gunshotSound = Dummy()
reloadSound = Dummy()
emptySound = Dummy()

def loadSounds():
	global gunshotSound, reloadSound, emptySound
	gunshotSound = pygame.mixer.Sound( os.path.join(DataPath, "deagle-1.wav") )
	reloadSound = pygame.mixer.Sound( os.path.join(DataPath, "reload.wav") )
	emptySound = pygame.mixer.Sound( os.path.join(DataPath, "empty.wav") )
	
def degClean(d):
	deg = d
	while deg < 0.0: deg += 360.0
	while deg > 360.0: deg -= 360.0
	return deg

def normVec(v):
	ln = sqrt( v[0]*v[0] + v[1]*v[1] )
	if ln == 0.0: return v
	return (v[0]/ln, v[1]/ln)

#########################
## Blood Particle System
#########################
class PlayerBloodSystem(BitmapParticleSystem):
	image = ( os.path.join(DataPath, BloodImg + ImgExt) )
	layer = "player"
	mutators = []

class BloodSystem(BitmapParticleSystem):
	image = ( os.path.join(DataPath, BloodImg + ImgExt) )
	layer = "effects"
	mutators = []
	
class BloodEmitter(PointEmitter):
	delay = 0.02
	num_emits = 5
	num_particles = 2
	life = 0.4
	fade_time = 0.4
	fade_in = 0
	scale = lazyRand(0.8, 1.2)
	scale_delta = -0.5
	rotation = lazyRand(-30.0, 30.0)
	rotation_delta = 15
	direction = lazyRand(150,210)
	color = (255,lazyRand(50),0,lazyRand(200,255))
	velocity = lazyRand(150,200)
	angle = 10

class PlayerBloodEmitter(PointEmitter):
	delay = 0.02
	num_emits = 5
	num_particles = 4
	life = 0.4
	fade_time = 0.4
	fade_in = 0
	scale = lazyRand(0.8, 1.2)
	scale_delta = -0.5
	rotation = lazyRand(-30.0, 30.0)
	rotation_delta = 15
	direction = lazyRand(0,360)
	color = (255,lazyRand(50),0,lazyRand(200,255))
	velocity = lazyRand(150,200)
	angle = 10

###################
## Story scroller
###################

def putStory(story, font, stop=110, sheight = 140, fadeList=None):
	# Story
	xx, yy = screen_w/2.0, screen_h * 0.333
	
	StoryTop = yy+stop
	if screen_w == 1024: StoryTop += 90
		
	StoryBottom = StoryTop + sheight
	
	StoryHeight = sheight
	ScrollSpeed = 0.1 # storyheight / second
	LineSpacing = 0.2
	
	tabx = 75
	if screen_w == 1024: tabx = 220
		
	# Bars for story fading
	Entity(os.path.join(DataPath, "scrolltop" + ImgExt)).place("bar").set(centerx=xx, y=StoryTop)
	Entity(os.path.join(DataPath, "scrollbottom" + ImgExt)).place("bar").set(centerx=xx, y=StoryBottom)
	
	i = 0
	for s in story:
		t = TextEntity(font, s).place("text").set(x=tabx,centery=StoryBottom )
		t.do(
			Delay(2.0 + 1.0 / ScrollSpeed * LineSpacing * i) + 
			Move(0, -ScrollSpeed * StoryHeight).limit(time=1.0/ScrollSpeed) + 
			Delete )
		if fadeList:
			fadeList.append( t )
		i += 1


##############
# Mouse
##############

class Mouse(Entity):
	def __init__(self):
		Entity.__init__(self, os.path.join(DataPath, "mouse" + ImgExt) )
		self.place("gui")
	
	def setpos(self, pos):
		self.x, self.y = pos


###########
## HP
###########
class Hp(Entity):
	def __init__(self, avatar):
		Entity.__init__(self, os.path.join(DataPath, HpImg + ImgExt) )
		self.avatar = avatar
		
		xx = screen_w / 2.0
		yy = screen_h - 40
		
		self.set(x=xx-20, y=yy)
		self.place("gui")
		
		# Fonts
		self.font = GLFont( ("arial", 20) )
		self.text = TextEntity(self.font, str(int(self.avatar.hp))).place("gui").set(centerx=xx+20,centery=yy+2)
	
	def update(self):
		self.text.set_text( str(int(self.avatar.hp)) )

#############
# Avatar
#############

class Avatar:
	
	# Name is the prefix of image filenames
	def __init__(self, name, world, x, y, radius, hp=100, angle=0.0, speed=10, wspeed=60.0, layer="entities"):
		self.name = name
		self.world = world
		
		# Health
		self.hp = hp
		
		# Positioning and moving
		self.radius = radius
		self.moving = False
		self.angle = angle
		self.speed = speed * pixelratio
		self.wspeed = wspeed
		self.x = x
		self.y = y
		
		# Animations
		self.bloodNode = None
		self.layer = layer
		self.loadAnims()
		self.anim = NumAnims-1
		
		self.timeToNextAnim = NormalSpeed / self.speed / 4.0
		self.nextAnim()
		
	
	def loadAnims(self):
		self.moveAnim = []
		for i in xrange(NumAnims):
			self.moveAnim.append ( Entity( os.path.join(DataPath, self.name + MovePostfix + str(i) + ImgExt) ) )
		
		self.dieAnim =  Entity( os.path.join(DataPath, self.name + DiePostfix + ImgExt ) )
	
	def calcPos(self, dt):
		# To be overwritten by subclass
		self.x += self.speed * dt
	
	def move(self, dt):
		if self.hp > 0:
			if self.moving:
				# Animation
				self.timeToNextAnim -= dt
				if self.timeToNextAnim <= 0:
					self.nextAnim()
	
					self.timeToNextAnim = NormalSpeed / self.speed / 4.0
			
				# Position calculation
				self.calcPos(dt)
			
			# Set angle and position of the entity
			self.moveAnim[self.anim].set(x=self.x, y=self.y, angle=self.angle)
		
		#BloodNode update
		self.dieAnim.set(x=self.x, y=self.y, angle=self.y)
	
	def hit(self, dhp):
		# Hit by dhp
		self.hp -= dhp
		
		if self.hp <= 0:
			# Disable animation, and make died
			self.moveAnim[self.anim].detach()
			self.dieAnim.set(x=self.x, y=self.y, angle=self.angle)
			self.dieAnim.place("corpses")
			
			# Remove moving sequence entities
			for anim in self.moveAnim:
				anim.delete()
	
	def nextAnim(self):
		# Hide last image
		self.moveAnim[self.anim].detach()
		
		self.anim += 1
		if self.anim == NumAnims:
			self.anim = 0
		
		# Re-appear
		self.moveAnim[self.anim].place(self.layer)

#################
## Monster
#################
class Monster(Avatar):
	def __init__(self, name, world, x, y, radius, damage=10, hp=100, angle=0.0, speed=10, wspeed=60.0, layer="entities"):
		Avatar.__init__(self, name, world, x, y, radius, hp, angle, speed, wspeed, layer)
		
		self.radius = radius
		self.damage = damage
		
		self.real_angle = random.uniform(0.0, 360.0)
		self.angle = self.real_angle
		self.moving = True
	
	def hit(self, dhp):
		Avatar.hit(self, dhp)
		
		# Bleed
		blood = Entity().set(realx=self.x, realy=self.y, angle=self.angle)
		self.world.bloodsystem.new_emitter( BloodEmitter, blood )
				
		if self.hp < 0:
			# Delayed diappear and deletion
			self.dieAnim.do(Delay(CorpseDelay) + AlphaFade(0, AlphaFadeTime) + CallFunc(GameScene.delMonster, self.world, self))
			self.world.monstersLive -= 1
	
	def calcPos(self, dt):
		# Get angle and distance to player
		pl = self.world.player
		dx = pl.x - self.x
		dy = pl.y - self.y
		
		plangle = degrees( atan2(dy, dx) )
		pldist = sqrt(dx*dx + dy*dy)
		
		# Angle to player
		dangle = degClean( plangle - 90.0 - self.real_angle )
		
		# Turn to player angle with wspeed
		if dangle > 180.0:	self.real_angle += self.wspeed * dt
		else: self.real_angle -= self.wspeed * dt
		
		# Approach, if not already near
		if pldist > self.radius + pl.radius:
			# Norm distance
			dx, dy = normVec((dx, dy))
			self.x += self.speed * dx * dt
			self.y += self.speed * dy * dt
		else:
			# If near, hurt him
			pl.hit(self.damage * dt)
		
		self.angle = self.real_angle
		
		
		


#################
## Player
#################

class Player(Avatar):
	def __init__(self, name, world, x, y, radius, hp=100, angle=0.0, speed=10, wspeed=60.0, layer="entities"):
		Avatar.__init__(self, name, world, x, y, radius, hp, angle, speed, wspeed, layer)
		
		self.moveFwd = False
		self.moveBack = False
		self.moveLeft = False
		self.moveRight = False
		
		self.diagMove = False
		
		# Bleeding
		self.blood = False
		
		# Look direction
		self.dx = 1
		self.dy = 0
	
	# Set look direction to pos
	def face(self, pos):
		self.dx, self.dy = normVec((pos[0] - self.x, pos[1] - self.y))
		
		self.angle = degrees( atan2(self.dy, self.dx) ) + 90
	
	def hit(self, dhp):
		# Hit by dhp
		Avatar.hit(self, dhp)
		
		def noBlood():
			self.blood = None
					
		if self.hp <= 0:
			self.hp = 0
# 			# Signal game over, etc.
			# Fade everything to black slowly, and change scene
			def nextScene():
				director.set_scene(DiedScene, self.world.stats)
			self.world.darkness.do(AlphaFade(255, 3.0) + CallFunc(nextScene))
		else:
			# Bleed
			if not self.blood:
				self.blood = Entity().set(realx=self.x, realy=self.y, angle=self.angle)
				self.world.pbloodsystem.new_emitter( PlayerBloodEmitter, self.blood )
				self.blood.do(AlphaFade(0, 0.2) + CallFunc(noBlood))
		
	
	
	def moveUpdate(self):
		horiz = self.moveLeft or self.moveRight
		vert = self.moveFwd or self.moveBack
		self.moving = horiz or vert
		self.diagMove = horiz and vert
	
	def setFwd(self, b):
		self.moveFwd = b
		self.moveUpdate()
	
	def setBack(self, b):
		self.moveBack = b
		self.moveUpdate()
	
	def setLeft(self, b):
		self.moveLeft = b
		self.moveUpdate()
	
	def setRight(self, b):
		self.moveRight = b
		self.moveUpdate()
	
	def calcPos(self, dt):
		
		if self.diagMove: mul = 0.707106
		else: mul = 1.0
		
		if self.moveFwd: self.y -= self.speed * dt * mul
		elif self.moveBack: self.y += self.speed *  dt * mul
		
		if self.moveLeft: self.x -= self.speed * dt * mul
		elif self.moveRight: self.x += self.speed * dt * mul
					
		# Clip coords to screen
			
#############
## MonsterFactory
#############
class MonsterFactory:
	def __init__(self, world, spawn_gauss):
		self.world = world
		self.spawn_gauss = spawn_gauss
		
		self.frozen = False
		self.next_spawn = world.end_time + random.gauss(spawn_gauss[0], spawn_gauss[1])
		
		## Description of monsters - (name, radius, speed, speedvariance, wspeed, wspeedvariance)
		spider1 = ("spider1", 30, 40, 10, 60, 10)
		snake = ("snake", 30, 60, 10, 80, 10)
		denever = ("denever", 30, 70, 10, 100, 15)
		
		self.monsterType = [ spider1, snake, denever ]
	
	def freeze(self):
		# No more change or monster output from now on
		self.frozen = True
	
	def update(self, dt):
		if self.frozen: return
		
		# Increase spawn rate (dt is seconds)
		rate, var = self.spawn_gauss
		new_rate = rate - dt*RateTimeDecrease
		self.spawn_gauss = ( new_rate , new_rate/2.0)
	
	def spawn(self):
		if self.frozen: return
		
		# If time to spawn, do it
		if self.next_spawn <= self.world.end_time:
			# Random monster type
			t = self.monsterType[ random.randint(0, len(self.monsterType)-1 ) ]
				
			# Random monster position
			i = random.randint(0, len(monsterStartPos)-1)
			xx, yy = monsterStartPos[i]
			
			# Spawn
			sp = random.gauss(t[2], t[3])
			wsp = random.gauss(t[4], t[5])
			monster = Monster(name=t[0], world=self.world, x=xx, y=yy, radius=t[1], speed=sp, wspeed=wsp )
			self.world.monsters.append( monster )
			self.world.monstersLive += 1
			
			# Next spawn
			self.next_spawn = self.world.end_time + random.gauss(self.spawn_gauss[0], self.spawn_gauss[1])
			
		
#############
## Stats
############
class Stats:
	def __init__(self):
		self.hits = 0
		self.shoots = 0
	
	def addShoot(self):
		self.shoots += 1
	
	def addHit(self):
		self.hits += 1
			

#############
## EndScenes
#############
class DiedScene(Scene):
	def enter(self, stat):
		# Print player statistics
		self.new_layer("text")
		self.new_layer("bar")
		
		font1 = GLFont( ("times", 24) )
		font2 = GLFont( ("arial", 18) )
		
		xx, yy = screen_w/2.0, screen_h * 0.333
		
		story=[
		"Here I lie in my own blood..",
		"",	"",	"",
		"Crushed",
		"",	"",
		"Defeated",
		"",	"",	"",
		"As I slowly die, a final tought fills my last second with joy."
		"","","",
		"Along with me, the monsters would finally disappear.."
		]
		putStory(story, GLFont(("times", 18)), stop=0 )
		
		try:
			ratio = stat.hits / float(stat.shoots) * 100.0
		except:
			ratio = 0
		statistic = "Your hit ratio is %d%%. Press ESC to exit, any other key to restart" % ratio
		t2 = TextEntity(font2, statistic).place("text").set(centerx=xx,centery=screen_h - 140)
		t2.set_alpha(0)
		t2.do(AlphaFade(255, 2.0))
		
	
	def handle_keydown(self, event):
		# Esc exits, any other key restarts
		if event.key == K_ESCAPE: director.quit()
		else: director.set_scene(GameScene)
	
	
class WonScene(Scene):
	def enter(self, stat):
		# Print player statistics
		self.new_layer("text")
		self.new_layer("bar")
		font1 = GLFont( ("times", 24) )
		font2 = GLFont( ("arial", 16) )
		
		xx, yy = screen_w/2.0, screen_h * 0.333
		
		story = [
		"Covered with heavy blood, corpses all around me. Finally, the",
		"mindless massacre is all over. The fear is over.",
		"","",
		"I can't realize it yet.",
		"", "", "",
		"It's silence. As I look around, I see the heaps of bodies",
		"fading, just like the blood on my hands. I was able to overcome",
		"my horror, and now they fade away..",
		"",	"",
		"I should be happy.",
		"", "",
		"But as I look around, I see nothing. This nothing is all I",
		"have now. Fighting my fear led me nowhere."
		"", "", "",
		"Fear is not worth to fight against.",
		"Fear deserves only to be forgotten.",
		"", "", "",
		"From now on, I won't fear anything."
		]
		putStory(story, GLFont(("times", 18)), stop=0 )
		
		try:
			ratio = stat.hits / float(stat.shoots) * 100.0
		except:
			ratio = 0
		
		thankyou = "Thank You for playing Darkest Darkness from Mindless Game Studio :)"
		statistic = "You killed all the bastards, your hit ratio is %d%%. Press ESC to exit." % ratio
		
		t2 = TextEntity(font2, statistic).place("text").set(centerx=xx,centery=screen_h - 100)
		t2.set_alpha(0)
		t2.do(AlphaFade(255, 2.0))
		
		t3 = TextEntity(font2, thankyou).place("text").set(centerx=xx,centery=screen_h - 140)
		t3.set_alpha(0)
		t3.do(AlphaFade(255, 2.0))
	
	def handle_keydown(self, event):
		# Esc exits game
		if event.key == K_ESCAPE: director.quit()


#############
## GameScene 
#############

class GameScene(Scene):
	def enter(self):
		self.start_time = pygame.time.get_ticks()
		self.last_time = self.start_time
		self.end_time = self.start_time
		self.reload_time = 0	# When to reload
		
		self.new_static("ground")
		self.new_stabile("corpses")
		self.new_stabile("entities")
		self.new_layer("effects")
		self.new_layer("darkness")
		self.new_stabile("player")
		self.new_layer("gui")
		
		# Monster and player bleeding
		self.bloodsystem = BloodSystem()
		self.pbloodsystem = PlayerBloodSystem()
		
		self.mousepos = (screen_w/2.0, screen_h/2.0)
		
		# Statictics
		self.stats = Stats()
		
		# Ground
		ground = pygame.Surface((screen_w, screen_h))
		gnd = pygame.image.load( os.path.join(DataPath, "ground" + ImgExt) )
		for pos in ((0,0), (640, 0), (0, 480), (640, 480)):
			ground.blit(gnd, pos)
		self.ground = Entity( ground ).place("ground").set(x=screen_w/2.0, y=screen_h/2.0)
		
		# Darkness
		dark = pygame.Surface((screen_w, screen_h))
		dark.fill((10,10,0,255))
		self.darkness = Entity( dark ).place("darkness").set(x=screen_w/2.0, y=screen_h/2.0)
		self.flash_times = [] # The recent times of shooting, when screen should go bright
		
		# Bulelt meter
		self.bulletEntities = []
		for i in range(0, MaxBullets*20, 20) :
			e = Entity(os.path.join(DataPath, BulletImg + ImgExt)).place("gui").set(x=40+i, y=screen_h-40)
			self.bulletEntities.append( e )
			
		self.bullets = MaxBullets
		
		# Player and gui elements
		self.player = Player(name="player", layer="player", world=self, x=screen_w/2.0, y=screen_h/2.0, radius=25, speed=120)
		self.hplevel = Hp(self.player)
		self.mouse = Mouse()
		
		# Monsters
		self.monsterFactory = MonsterFactory(self, (StartSpawnRate, StartSpawnRate/2.0) )
		self.monsters = []
		self.monstersLive = 0
		
	def delMonster(self, monster):
		monster.dieAnim.delete()
		self.monsters.remove( monster )
			
 	def shoot(self):
		if self.player.hp <= 0:
			# No shooting now..
			return
		
		origBullets = self.bullets
		
		if self.bullets > 0:
			self.stats.addShoot()
			gunshotSound.play()
			self.bullets -= 1
			self.bulletEntities[ self.bullets ].detach()
		
		if (self.bullets == 0) and (self.reload_time < self.end_time) :
			self.reload_time = self.end_time + ReloadTime
			self.player.moveAnim[0].do( CallFunc(reloadSound.play) )
		
		if origBullets == 0:
			emptySound.play()
			return
		
		# Check if we hit any monster
		# And keep track of the one with the shortest distance, ans its delta angle/max delta angle
		dfi = 0		# Difference between player angle and monster sight angle
		maxdfi = 0	# If dfi is greater then this, no hit occurs
		distance = 99999.0	# Monster distance
		hitmonster = None
		
		for m in self.monsters:
			# get distance, angles..
			dx,dy = m.x - self.player.x, m.y - self.player.y
			dist = sqrt(dx*dx + dy*dy)
			
			monsterdeg = degrees( atan2(dy, dx) )
			tdf = degClean( monsterdeg - (self.player.angle - 90.0) )
			df = min(tdf, 360.0 - tdf)
			
			# Because asin is not 1-1 relational, we need to play with 90degs..
			try:
				maxdf = 90.0 - abs (degrees ( asin( m.radius / dist ) ) - 90.0)
			except ValueError:
				# If monsters are too near, m.radius/dist becomes greater than 1.0
				maxdf = 45.0
			
			
			
			if (df < maxdf) and (dist < distance) and (m.hp > 0):
				# Hit someone nearer
				hitmonster = m
				distance = dist
				dfi, maxdfi = df, maxdf
		
		# Reapper the Things!!
		self.flash_times.append( self.end_time )
				
		# Hit that monster
		if hitmonster:
			self.stats.addHit()
			damage = BulletDamage * (1 - dfi/maxdfi)
			hitmonster.hit( damage )	
	
	def recalcDarkness(self):
		# Remove timed-out flashed
		while (len(self.flash_times) > 0) and (self.flash_times[0] + FlashFadeout < self.end_time):
			self.flash_times = self.flash_times[1:]
		
		def flashFunc(t):
			if t <= 100.0:	return t/100.0
			elif t <= 300.0:	return 1.0
			elif t <= 600.0:	return 1.0 - (t-300.0)/300.0
			else: return 0.0
		
		try:
			light = max( map ( flashFunc, map( lambda t: self.end_time - t, self.flash_times ) ) )
		except ValueError:
			light = 0.0
		self.darkness.set_alpha( 230 * (1.0 - light) )
	
	def realtick(self):
		# CPU nice
		pygame.time.wait(5)
		
		# Keep account of time
		self.end_time = pygame.time.get_ticks()
		dt = (self.end_time - self.last_time) / 1000.0
		self.last_time = self.end_time
		
		# Update monster factory spawn rate
		self.monsterFactory.update(dt)
		
		# If spawn rate reaches WinSpawnRate, then
		# stop spawning, and wait for victory
		if self.monsterFactory.spawn_gauss[0] <= WinSpawnRate:
			self.monsterFactory.freeze()
			if self.monstersLive == 0:
				# Won, congrats ;)
				def wonScene():
					director.set_scene(WonScene, self.stats)
				for i in xrange(NumAnims):
					self.player.moveAnim[i].do(AlphaFade(0, 3.0))
				self.darkness.do(AlphaFade(255, 3.0) + CallFunc(wonScene))
				
		
		# Spawn 'em
		self.monsterFactory.spawn()
				
		# Reload if required and time
		if (self.bullets == 0) and (self.end_time >= self.reload_time):
			self.bullets = MaxBullets
			for i in xrange(MaxBullets): self.bulletEntities[i].place("gui")
		
		# Fce player to mouse and move
		self.player.face(self.mousepos)
		self.player.move(dt)
		
		# Move monsters
		for m in self.monsters:
			m.move(dt)
			
		# Set darkness level
		self.recalcDarkness()
			
		# Update GUI
		self.hplevel.update()
			
	def handle_keydown(self, event):
		if event.key == K_w: self.player.setFwd(True)
		elif event.key == K_s: self.player.setBack(True)
		elif event.key == K_a: self.player.setLeft(True)
		elif event.key == K_d: self.player.setRight(True)
		
		elif event.key == K_ESCAPE: director.quit()
	
	def handle_keyup(self, event):
		if event.key == K_w: self.player.setFwd(False)
		elif event.key == K_s: self.player.setBack(False)
		elif event.key == K_a: self.player.setLeft(False)
		elif event.key == K_d: self.player.setRight(False)
	
	def handle_mousemotion(self, event):
		self.mousepos = event.pos
		self.mouse.setpos(event.pos)
	
	def handle_mousebuttondown(self, event):
		self.shoot()

################
## MenuScene
################

class MenuScene(Scene):
	def enter(self):
		# Print player statistics
		self.new_layer("text")
		self.new_layer("bar")
		font1a = GLFont( ("times", 24) )
		font1b = GLFont( ("times", 18) )
		font2 = GLFont( ("arial", 14) )
		
		xx, yy = screen_w/2.0, screen_h * 0.333
		
		# Intro screen
		txt = []		
		txt.append( TextEntity(font1a, "Darkest Darkness").place("text").set(centerx=xx,centery=yy) )
		txt.append( TextEntity(font1b, "by Mindless Game Studio").place("text").set(centerx=xx,centery=yy+40) )
		
		
		# Scrolling intro story
		story = [
		"In my childhood, just like other children, I feared the monsters",
		"lurking behind the door, down in the dark basement.",
		"",
		"Children however, wear off this fear as they get grown up.",
		"",
		"",
		"",
		"I could not. My mind was in constant terror, and I was not able",
		"to distract myself from that door.. My strong belief in the monsters",
		"called them to existence.",
		"",
		"They began weak, but as I heard them crawl behind that door..",
		"scream behind that door.. my stressed soul was tossed more",
		"and more into despair.",
		"",
		"I knew that that door of the basement, the door on my mind",
		"can't hold forever..",
		"",
		"",
		"I always wished that the monsters would disappear..",
		"but not this way.. not this way.."
		"",
		"",
		"",
		"This night.. it's time to counter my fears.",
		]
		
		putStory(story, font1b, fadeList=txt)
		
		txt.append( TextEntity(font2, "Press any key to begin your nightmare.").place("text").set(centerx=xx,centery=screen_h - 30) )
		
		self.txt = txt
		
	def handle_keydown(self, event):
		# Esc exits game
		if event.key == K_ESCAPE: director.quit()
		
		def gameScene():
			director.set_scene(GameScene)	
		
		for txt in self.txt[1:]:
			txt.do( AlphaFade(0, 1.0) )
		self.txt[0].do( AlphaFade(0, 1.0) + CallFunc(gameScene) )
		

if sound:
	pygame.mixer.init()
	loadSounds()


if psyco_on:
	try:
		import psyco
		psyco.bind(GameScene.recalcDarkness)
		psyco.bind(GameScene.realtick)
		psyco.bind(Avatar.move)
		psyco.bind(Avatar.calcPos)
		psyco.bind(Monster.move)
		psyco.bind(Monster.calcPos)
		psyco.bind(Player.move)
		psyco.bind(Player.calcPos)
		psyco.bind(normVec)
	
		
	except ImportError:
		print "[warning] psyco is not installed, you may install it for speed-up"

	

screen.init( (screen_w, screen_h), title="Darkest Darkness // MindlessGS // PyWeek #3", fullscreen=fs )
director.run(MenuScene)